-----------------------------------
-- Setting up scope and libs
-----------------------------------

local AddonName = "iMoney"
iMoney = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0");

local QTip = LibStub("LibQTip-1.0");
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

local Tooltip;

------------------------------------------
-- Variables, functions and colors
------------------------------------------

local COLOR_GOLD = "|cfffed100%s|r";
local COLOR_RED  = "|cffff0000%s|r";
local COLOR_GREEN= "|cff00ff00%s|r";

local Gold = 0;
local OldGold = 0;
local CharName = GetUnitName("player", false);

local ClassColors = {};
for k in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[k] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end

local function tclear(t, wipe)
	if( type(t) ~= "table" ) then return end;
	for k in pairs(t) do
		t[k] = nil;
	end
	t[''] = 1;
	t[''] = nil;
	if( wipe ) then
		t = nil;
	end
end

-----------------------------
-- Setting up the feed
-----------------------------

local function Feed_OnEnter(anchor)
	for k, v in QTip:IterateTooltips() do
		if( type(k) == "string" and strsub(k, 1, 6) == "iSuite" ) then
			v:Release(k);
		end
	end
	
	Tooltip = QTip:Acquire("iSuite"..AddonName);
	Tooltip:SetColumnLayout(2, "LEFT", "RIGHT");
	Tooltip:SetAutoHideDelay(0.1, anchor);
	Tooltip:SmartAnchorTo(anchor);
	iMoney:UpdateTooltip();
	Tooltip:Show();
end

iMoney.Feed = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",
	OnEnter = Feed_OnEnter,
});

----------------------
-- Initializing
----------------------

function iMoney:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("iMoneyDB", {realm={today="",chars={}}}, true).realm;
	
	-- character currently missing in the table? no problem!
	if( not self.db.chars[CharName] ) then
		local _, class = UnitClass("player");
		self.db.chars[CharName] = {
			gold = 0,
			gold_in = 0,
			gold_out = 0,
			class = class,
			--today = 0,
		};
	end
	
	self:RegisterEvent("PLAYER_MONEY", "UpdateMoney", false);
end

function iMoney:OnEnable()
	self:UpdateMoney(true); -- we just can call GetMoney() on Enable, that's why this call is here
	
	-- for new chars with >0 gold, we need to init the today value with the current gold amount
	local c = self.db.chars[CharName];
	--if( c.today == 0 and c.today ~= c.gold ) then
	--	c.today = c.gold;
	--end
	
	local today = date("%y-%m-%d");
	if( today ~= self.db.today ) then
		self.db.today = today;
		
		for k, v in pairs(self.db.chars) do
			--v.today = v.gold;
			v.gold_in = 0;
			v.gold_out = 0;
		end
	end
end

------------------------------------------
-- MoneyString and UpdateMoney
------------------------------------------

local function MoneyString(gold, encolor)
	local str;

	if( encolor ) then
		if( gold < 0 ) then
			str = (COLOR_RED):format("-"..GetCoinTextureString(-gold, 0));
		elseif( gold > 0 ) then
			str = (COLOR_GREEN):format(GetCoinTextureString(gold, 0));
		else
			str = GetCoinTextureString(gold, 0);
		end
	else
		if( gold < 0 ) then
			str = "-"..GetCoinTextureString(-gold, 0);
		else
			str = GetCoinTextureString(gold, 0);
		end
	end
	
	return str;
end

function iMoney:UpdateMoney(firstCheck)
	local money = GetMoney();
	
	-- in an older version, iMoney did not have this feature. so we must check if the config values are there.
	if( self.db.chars[CharName].gold_in == nil ) then self.db.chars[CharName].gold_in = 0 end
	if( self.db.chars[CharName].gold_out == nil ) then self.db.chars[CharName].gold_out = 0 end
	
	if( not firstCheck ) then
		if( money > Gold ) then
			self.db.chars[CharName].gold_in = self.db.chars[CharName].gold_in + (money - Gold);
		else
			self.db.chars[CharName].gold_out= self.db.chars[CharName].gold_out + (Gold - money);
		end
	end
	Gold = money;
	
	self.db.chars[CharName].gold = money;
	self.Feed.text = MoneyString(money);
end

------------------------------------------
-- UpdateTooltip
------------------------------------------

local function iMoneyClickLine(_, name, button)
	if( button == "RightButton" and name ~= CharName ) then
		iMoney.db.chars[name] = nil;
		iMoney:UpdateTooltip();
	end
end

local function iMoneySort(a, b)
	return a.gold > b.gold;
end

local SortingTable = {};
function iMoney:UpdateTooltip()
	Tooltip:Clear();
	
	Tooltip:AddLine(
		(COLOR_GOLD):format(L["Today Session"]),
		MoneyString(self.db.chars[CharName].gold_in - self.db.chars[CharName].gold_out, true)
	);
	Tooltip:AddSeparator();
	Tooltip:AddLine(L["Gains"], MoneyString(self.db.chars[CharName].gold_in, true));
	Tooltip:AddLine(L["Losses"], MoneyString(-self.db.chars[CharName].gold_out, true));
	Tooltip:AddLine(" ");
	
	local total = 0;
	
	for k, v in pairs(self.db.chars) do
		v.name = k;
		table.insert(SortingTable, v);
	end
	table.sort(SortingTable, iMoneySort);
	
	for i = 1, #SortingTable do
		local line = Tooltip:AddLine(
			("%s%s|r"):format(ClassColors[SortingTable[i].class], SortingTable[i].name),
			MoneyString(SortingTable[i].gold)
		);
		total = total + SortingTable[i].gold;
		
		Tooltip:SetLineScript(line, "OnMouseDown", iMoneyClickLine, SortingTable[i].name);
	end
	
	tclear(SortingTable);
	
	Tooltip:AddLine(" ");
	Tooltip:AddLine(
		(COLOR_GOLD):format(L["Total Gold"]),
		MoneyString(total)
	);
end